#!/usr/bin/python3
# -*- coding: utf-8 -*-
# nemo-pastebin - Nemo extension to paste a file to a pastebin service
# Written by:
#    Alessio Treglia <quadrispro@ubuntu.com>
# Copyright (C) 2009-2010, Alessio Treglia
#
# This package is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This package is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this package; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA
#

import os
import sys
import time
import gettext
import re
from urllib import parse
import webbrowser
from subprocess import check_output, CalledProcessError
from threading import Thread
import signal
signal.signal(signal.SIGINT, signal.SIG_DFL)
import gi
gi.require_version("Gtk", "3.0")
from gi.repository import GObject, Gdk, Gio, Gtk
gi.require_version('Nemo', '3.0')
from gi.repository import Nemo

try:
    gi.require_version('Notify', '0.7')
    from gi.repository import Notify
except ImportError:
    Notify = None

# Globals
APP_NAME = "nemo-pastebin"
LOGGING = 1
MAINTAINER_MODE = True

gettext.install("nemo-extensions")

class log:
    def log(self, message):
        if self.logger:
            self.logger.debug(message)
    def __init__(self):
        logger = None
        if LOGGING:
            import logging
            logger = logging.getLogger(APP_NAME)
            logger.setLevel(logging.DEBUG)
            ch = logging.StreamHandler()
            ch.setLevel(logging.DEBUG)
            logger.addHandler(ch)
        self.logger = logger

log = log().log

Notify.init("Nemo pastebin extension")

class PastebinThread(Thread):
    def __init__(self, settings, filename):
        self.settings = settings
        self.filename = filename
        Thread.__init__(self)

    def run(self):
        log ("PastebinThread started!")
        cmdline = ["pastebinit"]

        options = {}
        options['-b'] = "http://" + self.settings.get_string("pastebin")
        options['-a'] = self.settings.get_string("author")
        options['-u'] = self.settings.get_string("username")
        options['-p'] = self.settings.get_string("password")

        if not options['-a'] or options['-a'].strip() == '':
            options['-a'] = os.getenv("USERNAME")

        for opt in options:
            if options[opt] != None and options[opt].strip() != '':
                cmdline.append(opt)
                cmdline.append(options[opt])

        cmdline.append(self.filename)

        summary = ''
        message = ''
        icon = None
        helper = Gtk.Button()

        try:
            pasteurl = check_output(cmdline).decode().strip()
        except CalledProcessError as error:
            # error.cmd and error.returncode, pasteurl has strerr
            summary = error.message
            icon = helper.render_icon(Gtk.STOCK_DIALOG_ERROR, Gtk.IconSize.DIALOG, None)
        except:
            #        if not pasteurl or not re.match("^http://.*$", pasteurl):
            summary = _("Unable to read or parse the result page.")
            message = _("It could be a server timeout or a change server side. Try later.")
            icon = helper.render_icon(Gtk.STOCK_DIALOG_ERROR, Gtk.IconSize.DIALOG, None)
        else:
            summary = 'File pasted to: '
            #URLmarkup = '<a href="%(pasteurl)s">%(pasteurl)s</a>'
            #message = URLmarkup % {'pasteurl':pasteurl}
            message = pasteurl
            icon = helper.render_icon(Gtk.STOCK_PASTE, Gtk.IconSize.DIALOG, None)

            atom = Gdk.atom_intern('CLIPBOARD', True)
            cb = Gtk.Clipboard.get(atom)
            cb.clear()
            cb.set_text(pasteurl, -1)

            # Open a browser window
            if self.settings['openbrowser']:
                webbrowser.open(pasteurl)

        # Show a bubble
        if self.settings['shownotification']:
            if Notify != None:
                n = Notify.Notification.new(summary, message, None)
                n.set_image_from_pixbuf(icon)
                n.show()
            else:
                print("libnotify is not installed")

class PastebinitExtension(GObject.GObject, Nemo.MenuProvider, Nemo.NameAndDescProvider):
    BASE_KEY = "apps.nemo-pastebin"
    def __init__(self):
        log ("Initializing nemo-pastebin extension...")
        self.settings = Gio.Settings.new(self.BASE_KEY)
        self.validate_settings()

    def validate_settings(self):
        # Make sure "pastebin" value is supported by pastebinit
        pass #TODO

    def show_error_message(self, error, description):
        n = Notify.Notification.new(error, description, None)
        helper = Gtk.Button()
        icon = helper.render_icon(Gtk.STOCK_DIALOG_ERROR, Gtk.IconSize.DIALOG)
        n.set_icon_from_pixbuf(icon)
        n.show()

    def get_file_items(self, window, files):
        if len(files)!=1:
            return
        filename = files[0]

        if filename.get_uri_scheme() != 'file' or filename.is_directory():
            return

        items = []
        pastebin = self.settings.get_string("pastebin")
        #Called when the user selects a file in Nemo.
        item = Nemo.MenuItem(name="NemoPython::pastebin_item",
                             label=_("Pastebin to %s") % pastebin,
                             tip=_("Send this file to %s") % pastebin)
        item.set_property('icon', "nemo-pastebin")
        item.connect("activate", self.menu_activate_cb, files)
        items.append(item)
        return items

    def get_background_items(self, window, item):
        return []

    def menu_activate_cb(self, menu, files):
        if len(files) != 1:
            return
        filename = files[0]

        # TODO: MIME check

        filename = parse.unquote(filename.get_uri()[7:])

        thread = PastebinThread(self.settings, filename)
        thread.start()
        while (thread.isAlive()):
            time.sleep(0.09)
            while Gtk.events_pending():
                Gtk.main_iteration()

    def get_name_and_desc(self):
        description = _("Send files to a paste service via the context menu")
        return [(f"nemo-pastebin:::{description}:::nemo-pastebin-configurator")]



#!/usr/bin/python3
# -*- coding: utf-8 -*-
# nemo-pastebin - Nemo extension to paste a file to a pastebin service
# Written by:
#    Alessio Treglia <quadrispro@ubuntu.com>
# Copyright (C) 2009-2010, Alessio Treglia
#
# This package is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This package is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this package; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA
#

import os
import sys
import re
import gettext
from subprocess import check_output

import gi
gi.require_version('Gtk', '3.0')
from gi.repository import Gio, Gtk

gettext.install("nemo-extensions")

def get_presets():
    try:
        res = check_output(["pastebinit", "-l"]).decode()
        # split the line and throw away the first slice
        res = re.split("\n-?\s?", res.strip())[1:]
    except OSError as e:
        print(e)
        sys.exit(1)
    except Exception as e:
        print(e)
        sys.exit(-1)
    return res

UI_FILE = '/usr/share/nemo-pastebin/nemo-pastebin-configurator.ui'

class Controller(object):
    BASE_KEY = "apps.nemo-pastebin"
    def __init__(self):
        self.settings = settings = Gio.Settings.new(self.BASE_KEY)
        builder = Gtk.Builder.new()
        builder.set_translation_domain("nemo-extensions")
        builder.add_from_file(UI_FILE)

        self.dialog_main = builder.get_object('dialog_main')
        tableOptions = builder.get_object('table_options')

        # Set checkbuttons up
        openbrowser = builder.get_object('openbrowser')
        shownotification = builder.get_object('shownotification')

        openbrowser.set_active(settings.get_boolean("openbrowser"))
        settings.connect("changed::openbrowser", self.on_openbrowser_changed, openbrowser)
        openbrowser.connect('toggled', self.on_openbrowser_toggled, settings)
        shownotification.set_active(settings.get_boolean("shownotification"))
        settings.connect("changed::shownotification", self.on_shownotification_changed, shownotification)
        shownotification.connect('toggled', self.on_shownotification_toggled, settings)

        # Setup entries
        self.author = author = builder.get_object('entry_author')
        self.username = username = builder.get_object('entry_username')
        self.password = password = builder.get_object('entry_password')

        author.set_text(settings.get_string("author"))
        username.set_text(settings.get_string("username"))
        password.set_text(settings.get_string("password"))

        # Set combobox up
        self.presets = presets = get_presets()
        pastebin = builder.get_object("combobox_pastebin")
        pastebin.set_entry_text_column(0)
        curr_selected = settings.get_string("pastebin")
        pastebin.insert_text(0, curr_selected)
        #presets.remove(curr_selected)
        for i in presets:
            if i != curr_selected:
                pastebin.append_text(i)
        pastebin.set_active(0)
        settings.connect("changed::pastebin", self.on_pastebin_changed, pastebin)
        pastebin.connect("changed", self.on_pastebin_combo_changed, settings)

        builder.connect_signals(self)
        self.dialog_main.connect('delete_event', self.quit)
        self.dialog_main.show_all()

    def on_openbrowser_changed(self, settings, key, check_button):
        check_button.set_active(settings.get_boolean("openbrowser"))
    def on_shownotification_changed(self, settings, key, check_button):
        check_button.set_active(settings.get_boolean("shownotification"))

    def on_openbrowser_toggled(self, button, settings):
        settings.set_boolean("openbrowser", button.get_active())
    def on_shownotification_toggled(self, button, settings):
        settings.set_boolean("shownotification", button.get_active())

    def on_pastebin_changed(self, settings, key, combo_box):
        pass
        #combo_box.set_active(self.presets.index(settings.get_string("pastebin")))
    def on_pastebin_combo_changed(self, combo_box, settings):
        print("Selected: %s" % combo_box.get_active_text())
        settings.set_string("pastebin", combo_box.get_active_text())

    def _save_remaining_settings(self):
        self.settings.set_string("author", self.author.get_text())
        self.settings.set_string("username", self.username.get_text())
        self.settings.set_string("password", self.password.get_text())

    def showErrorMessage(self, message):
        md = Gtk.MessageDialog.new(parent=self.dialog_main,
                                   message_format=message,
                                   buttons=Gtk.ButtonsType.CLOSE,
                                   type = Gtk.MessageType.ERROR,
                                   flags = Gtk.DialogFlags.MODAL | Gtk.DialogFlags.DESTROY_WITH_PARENT)
        md.run()
        md.destroy()

    def quit(self, *args):
        Gtk.main_quit()

    def on_button_close_clicked(self, widget, data=None):
        self._save_remaining_settings()
        self.dialog_main.destroy()
        self.quit()

def main():
    c = Controller()
    Gtk.main()

if __name__ == "__main__":
    main()
